package cz.muni.fi.pa165.movie;

import cz.muni.fi.pa165.common.BaseService;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class MovieService extends BaseService<MovieDo> {

    @Getter
    private final MovieRepository repository;

    @Getter
    private final MovieMapper movieMapper;

    /**
     * Constructs a new instance of the MovieService class.
     *
     * @param movieRepository The repository for performing CRUD operations on MovieDo entities.
     * @param movieMapper The mapper for converting Movie entities to and from DTOs.
     */
    @Autowired
    public MovieService(MovieRepository movieRepository, MovieMapper movieMapper) {
        this.repository = movieRepository;
        this.movieMapper = movieMapper;
    }

    /**
     * Returns a list of all movies.
     *
     * @return the list of all movies
     */
    @Transactional(readOnly = true)
    public List<MovieDto> getMovies () {
        return repository.findAll().stream().map(movieMapper::toDto).toList();
    }

    /**
     * Returns the movie with the specified ID.
     *
     * @param id the ID of the movie to be returned
     * @return the movie with the specified ID
     */
    @Transactional(readOnly = true)
    public MovieDto getMovie(long id) {
        MovieDo movie = repository.getReferenceById(id);
        return movieMapper.toDto(movie);
    }

    /**
     * Creates a new movie.
     *
     * @param movieDto the movie to be created
     */
    @Transactional
    public void createMovie(MovieDto movieDto) {
        MovieDo movie = movieMapper.fromDto(movieDto);
        repository.save(movie);
    }

    /**
     * Updates an existing movie.
     *
     * @param id the ID of the movie to be updated
     * @param movieDto the updated movie information
     */
    @Transactional
    public void updateMovie(long id, MovieDto movieDto) {
        movieMapper.updateFromDto(movieDto, repository.getReferenceById(id));
    }

    /**
     * Deletes the movie with the specified ID.
     *
     * @param id the ID of the movie to be deleted
     */
    @Transactional
    public void deleteMovie(long id) {
        repository.delete(repository.getReferenceById(id));
    }

    /**
     * Deletes all movies.
     */
    @Transactional
    public void deleteAllMovies() {
        repository.deleteAll();
    }

    /**
     * Returns the movie with the specified title.
     *
     * @param title the title of the movie to be returned
     * @return the movie with the specified title
     */
    @Transactional(readOnly = true)
    public MovieDto findByTitle(String title) {
        return movieMapper.toDto(repository.findByTitle(title));
    }
}
